options nodate nonumber;

data ACT;
input GROUP $1-10 ACT;
cards;
Control 	 63
Control 	 67
Control 	 54
Control 	 76
Control 	 61
DrugHigh    154
DrugHigh    173
DrugHigh    153
DrugHigh    150
DrugHigh    152
DrugLow     110
DrugLow     111
DrugLow      96
DrugLow      69
DrugLow     106
DrugMed      98
DrugMed      99
DrugMed      83
DrugMed     107
DrugMed      72
;
run;


%macro NPARMCC(DSN,GROUP,CONTROL,VAR,ALPHA);
%***************************************************************;
%* A Macro for Performing all Pair-wise Nonparametric          *;
%* Against a Common Control Group                              *;
%* ------------------------------------------------------------*;
%*                                                             *;
%* This macro is designed to perform all two-sided pair-wise   *;
%* comparisons of the location parameters for n groups (n gt 2)*;
%* against a designated control group in a one-way layout.     *;
%* The method for the procedure involves a large sample        *;
%* approximation. The method of comparison employs ranking all *;
%* non-missing observations and averaging the ranks for each   *;
%* level of the class variable (&GROUP).                       *;
%*                                                             *;
%* The required inputs for the macro are a SAS data set (&DSN) *;
%* containing a character class variable (&GROUP), a designated*;
%* level of the class variable serving as the control group    *;
%* (&CONTROL), a response variable (&VAR), and a family-wise   *;
%* error rate (&ALPHA).                                        *;
%*                                                             *;
%* For more details on this nonparametric multiple comparison  *;
%* procedure, see Hollander and Wolfe s Nonparametric          *;
%* Statistical Methods, 1/e pp. 130-131 and Miller s           *;
%* Simultaneous Statistical Inference, 2/e pp. 129-138.        *;
%*                                                             *;
%* Overall Macro Scheme:                                       *;
%* --------------------                                        *;
%* The macro consists of a body of code containing one embedded*;
%* macro (%GROUPS). The embedded macro determines the number of*;
%* groups present (&NGRPS). If a group in the SAS data set does*;
%* not contain at least one response value, it will not be     *;
%* included in the analysis. The embedded macro also creates   *;
%* one global macro variable that contains the group labels    *;
%* (&GRPVEC) for the levels of the class variable.             *;
%*                                                             *;
%* The main body of the SAS macro code determines summary      *;
%* statistics (e.g., average ranks, sample sizes, etc.) This   *;
%* information is then employed to calculate the pair-wise     *;
%* test statistics. The cutoff for the test statistic is       *;
%* calculated with PROBIT function. The results are then       *;
%* printed out with a PROC PRINT procedure.                    *;
%*                                                             *;
%*If you have questions about this code or its application,    *;
%*please do not hesitate to contact me.                        *;
%*                                                             *;
%*Paul Juneau                                                  *;
%*Associate Director                                           *;
%*Midwest Nonclinical Statistics - Michigan Laboratories       *;
%*Pfizer Global Research & Development                         *;
%*734-622-1791                                                 *;
%*paul.juneau@pfizer.com                                       *;
%*                                                             *;
%*2004                                                         *;
%***************************************************************;

%macro GROUPS;
%global GRPVEC;

proc sort data=&DSN; by &GROUP;

%*First, take the input data set &DSN and eliminate missing values;
data &GROUP; set &DSN; by &GROUP;
if first.&GROUP;
if &GROUP ne "&CONTROL";
run;

proc transpose data=&GROUP out=GRPVEC; var &GROUP;

%*Determine the number of levels of the class variable;
data _null_;
call symput('NGRPS',left(put(count,8.)));
stop;
set &GROUP nobs=COUNT;
run;

data null; set GRPVEC;

length GRPVEC $100;

%*Create a macro variable containing class levels starting with the 
designated control group;
GRPVEC=
%do G=1 %to %eval(&NGRPS-1);
 trim(left(COL&G))||" "||
%end;
 trim(left(COL&NGRPS));

GRPVEC="&CONTROL"||" "||GRPVEC;

call SYMPUT('GRPVEC',GRPVEC);
run;

%mend GROUPS;

%GROUPS

proc sort data=&DSN;
by &GROUP;

%*Rank all non-missing responses;
proc rank data=&DSN out=R&VAR;
ranks R&VAR;
var &VAR;
run;

proc sort data=R&VAR;
by &GROUP;
run;

%*Calculate the sum of ranks;
proc univariate noprint data=R&VAR;
var R&VAR;
by &GROUP;
output out=S&VAR sum=S&VAR n=N&VAR;
run;

%*Determine the total sample size for the entire study;
proc univariate noprint data=S&VAR;
var N&VAR;
output out=SN&VAR sum=SN&VAR;
run;

%*Create a macro variable for the total sample size;
data _null_;
 set SN&VAR;

call symput("N",left(put(SN&VAR,8.)));
run;

data SR&VAR; set S&VAR;
 if &GROUP ne "&CONTROL";
keep S&VAR;
run;

%*Create a macro variable for the sample size of the control group;
data _null_; set S&VAR;
 if &GROUP = "&CONTROL";
 call symput("NC",N&VAR);
run;

data N&VAR; set S&VAR;
 if &GROUP ne "&CONTROL";
keep N&VAR;
run;

proc transpose data=SR&VAR out=SRVEC; var S&VAR;
run;

data _null_;
 set S&VAR;
 if &GROUP = "&CONTROL";

 call symput("SRC",left(put(S&VAR,11.6)));

run;

%*Create a macro variable containing the summary statistics;
data _null_;
 set SRVEC;

length SRVEC $50;

SRVEC=
%do S=1 %to %eval(&NGRPS-1);
 trim(left(put(COL&S,11.3)))||" "||
%end;
 trim(left(put(COL&NGRPS,11.3)));

call SYMPUT('SRVEC',SRVEC);
run;

%let SRVEC=&SRC &SRVEC;

proc transpose data=N&VAR out=nVEC; var n&VAR;
run;

data _null_;
 set NVEC;

nVEC=
%do S2=1 %to %eval(&NGRPS-1);
 trim(left(put(COL&S2,8.)))||" "||
%end;
 trim(left(put(COL&NGRPS,8.)));

nVEC=&NC||" "||nVEC;

call SYMPUT('NVEC',NVEC);
run;

%*Perform pair-wise MCP;
data NPARMCP;

length DIFFLBL $50;

%do U=2 %to %eval(&NGRPS+1);

 %do V=1 %to 1;

  %let RSUM&U=%scan(&SRVEC,&U," ");
  %let N&U=%scan(&NVEC,&U," ");
  %let RSUM&V=%scan(&SRVEC,&V," ");
  %let N&V=%scan(&NVEC,&V," ");

  c+1;
  DIFFLBL="%scan(&GRPVEC,&U," ") - &CONTROL";
  AVDIFF=abs((&&RSUM&U/&&N&U)-(&&RSUM&V/&&N&V));
  DCUTOFF=probit(1-(&ALPHA/&NGRPS))
         *(((&N*(&N+1))/12)**0.5)
         *((1/&&N&U + 1/&&N&V)**0.5);
  MCUTOFF= probmc('MAXMOD',.,(1-&ALPHA),.,&NGRPS)
         *(((&N*(&N+1))/12)**0.5)
         *((1/&&N&U + 1/&&N&V)**0.5);
  SYMBOLD="    ";
  SYMBOLM="    ";
  if AVDIFF ge DCUTOFF then do;
   SYMBOLD="*D*";
  end;
  if AVDIFF ge MCUTOFF then do;
   SYMBOLM="*M*";
  end;
  output;
 %end;

%end;

label DIFFLBL="Group@Comparisons"
      AVDIFF="Difference@in@Average@Ranks"
      DCUTOFF="Dunn@Cutoff@at@Alpha=&ALPHA"
	  MCUTOFF="Miller@Cutoff@at@Alpha=&ALPHA"
      SYMBOLD="Significant@Difference@Dunn's = *D*"
	  SYMBOLM="Significant@Difference@Miller's = *M*"
      c="Comparison@Number"
;

title1 "Large Sample Approximation Multiple Comparison Procedure";
title2 "For All Treatment Compared Against a Control (Control Group = &CONTROL)";
title3 "%trim(%left(%eval(&NGRPS+1))) Groups: %trim(%left(&GRPVEC)) (Respective Sample Sizes: %trim(%left(&NVEC)))";
title4 "Alpha = &ALPHA";
title5 ;
title6 "Method Suggested by Dunn (1964) for UNBALANCED Sample Sizes";
title7 "Method Suggested by Miller (1966) for Balanced Sample Sizes";
%*Print out the results;
proc print label split='@' noobs;
run;

%mend NPARMCC;

%NPARMCC(ACT,GROUP,Control,ACT,0.05);





 

