options nodate nonumber;

data TRI;
input GROUP $1-12 TRI;
cards;
Vehicle   	 89	
Vehicle   	 77	
Vehicle   	105	
Vehicle   	119	
Vehicle     109	
TreatmentA	127	
TreatmentA	 71	
TreatmentA	 57	
TreatmentA	 83	
TreatmentA	 53	
TreatmentB	 37	
TreatmentB	 32	
TreatmentB	 61	
TreatmentB	 73	
TreatmentB	 42	
TreatmentC	 62	
TreatmentC	 76
TreatmentC	 70
TreatmentC	 43
TreatmentC	 44
;
run;

%macro DSCF(DSN,GROUP,VAR,ALPHA);

%***************************************************************;
%* The Dwass Steele Critchlow-Fligner Macro                    *;
%* ------------------------------------------------------------*;
%*                                                             *;
%* This macro is designed to perform all two-sided pair-wise   *;
%* comparisons of the location parameters for n groups (n gt 2)*;
%* in a one-way layout. The method for the procedure involves a*;
%* large sample approximation. The method of comparison employs*;
%* pairwise Wilcoxon statistics for all pairs of groups in the *;
%* the data. The macro also contain an adjustment for ties.    *;
%*                                                             *;
%* The required inputs for the macro are a SAS data set (&DSN) *;
%* containing a character class variable (&GROUP), a response  *;
%* variable (&VAR), and a family-wise error rate (&ALPHA).     *;
%*                                                             *;
%* For more details on this nonparametric multiple comparison  *;
%* procedure, see Hollander and Wolfe s Nonparametric          *;
%* Statistical Methods, 2/e pp. 240-248.                       *;
%*                                                             *;
%* Overall Macro Scheme:                                       *;
%* --------------------                                        *;
%* The macro consists of a body of code containing one embedded*;
%* macro (%GROUPS). The embedded macro determines the number of*;
%* groups present (&NGRPS). If a group in the SAS data set does*;
%* not contain at least one response value, it will not be     *;
%* included in the analysis. The embedded macro also creates   *;
%* two global macro variables that contain the group labels    *;
%* (&GRPVEC) for the levels of the class variable and          *;
%* information about the sample size for each group (&NVEC).   *;
%*                                                             *;
%* The main body of the SAS macro code determines summary      *;
%* statistics (e.g., Wilcoxon test statistics) and the number  *;
%* of ties present in the pair-wise comparison. This           *;
%* information is then employed to calculate the pair-wise     *;
%* test statistics. The cutoff for the test statistic is       *;
%* calculated with PROBMC function. As the macro iterates      *;
%* between all pair-wise comparisons it concantenates          *;
%* successive SAS data sets in a data set called STAT. The     *;
%* results are then printed out with a PROC PRINT procedure.   *;
%*                                                             *;
%*If you have questions about this code or its application,    *;
%*please do not hesitate to contact me.                        *;
%*                                                             *;
%*Paul Juneau                                                  *;
%*Associate Director                                           *;
%*Midwest Nonclinical Statistics - Michigan Laboratories       *;
%*Pfizer Global Research & Development                         *;
%*734-622-1791                                                 *;
%*paul.juneau@pfizer.com                                       *;
%*                                                             *;
%*2004                                                         *;
%***************************************************************;

%macro GROUPS;
%*This macro creates three outputs:                       *;
%*(1) The total number of groups present (&NGRPS)         *; 
%*(2) A macro variable that contains all of the individual*;
%*    class variable sample sizes (&NVEC)                 *;
%*(3) A macro variable that contains all of the individual*;
%*    class variable labels (&GRPVEC)                     *;

%global GRPVEC NVEC;

%*Remove missing response values;
data &DSN.2;
 set &DSN;
 if &VAR ne .;
run;

proc sort data=&DSN.2; by &GROUP;
run;

%*Identify levels of class variable;
data &GROUP; set &DSN.2; by &GROUP;
if first.&GROUP;
run;

proc transpose data=&GROUP out=GRPVEC; var &GROUP;
run;

%*Determine the total sample size for the experiment;
data _null_;
call symput('NGRPS',left(put(count,8.)));
stop;
set &GROUP nobs=COUNT;
run;

%*Determine the sample size for each level of the class variable (group);
proc univariate noprint data=&DSN.2;
var &VAR;
by &GROUP;
output out=N&VAR n=N&VAR;
run;

proc transpose data=N&VAR out=nVEC; var n&VAR;
run;

%*Create a macro variable that contains each class level s sample size
  for later processing;

data _null_;
 set NVEC;
nVEC=
%do S=1 %to %eval(&NGRPS-1);
 trim(left(put(COL&S,8.)))||" "||
%end;
 trim(left(put(COL&NGRPS,8.)));
call SYMPUT('NVEC',NVEC);
run;

%*Create a macro variable that contains a label for each level of the class
  variable;

data _null_; set GRPVEC;

GRPVEC=
%do G=1 %to %eval(&NGRPS-1);
 trim(left(COL&G))||" "||
%end;
 trim(left(COL&NGRPS));
call SYMPUT('GRPVEC',GRPVEC);
run;

%mend GROUPS;

%GROUPS

%*Start iterative processing with an empty data SAS data set called STAT;

data STAT;
run;

%*Start calculating pair-wise statistics necessary for the multiple comparison 
  procedure;

%do U=1 %to &NGRPS;

 %do V=%eval(&U+1) %to &NGRPS;

%*Determine sample sizes of two groups involved in comparisons;

 proc means data=&DSN.2 noprint n;
 by &GROUP;
 where &GROUP in ("%scan(&GRPVEC,&U," ")","%scan(&GRPVEC,&V," ")");
 var &VAR;
 output out=N&U&V(keep=N) n=N;
 run;

%*Calculate pair-wise Wilcoxon statistics for group U and group V;

 proc npar1way data=&DSN.2 wilcoxon noprint;
 where &GROUP in ("%scan(&GRPVEC,&U," ")","%scan(&GRPVEC,&V," ")");
 class &GROUP;
 var &VAR;
 output out=W&U&V(keep=_WIL_);
 run; 

%*Determine the number of ties in the original data set (with all
  missing values removed;

 proc freq data=&DSN.2 noprint;
 where &GROUP in ("%scan(&GRPVEC,&U," ")","%scan(&GRPVEC,&V," ")");
 tables &VAR/out=T&U&V(keep=COUNT);
 run;

%*Calculate tie-adjustment factor necessary for variance estimate in
  test statistic;

 data T&U&V;
  set T&U&V;

  TIE=(COUNT-1)*(COUNT)*(COUNT+1);

 run;

%*Calculate the sum value of tie-adjustment value;

 proc means data=T&U&V sum noprint;
  var TIE;
  output out=TSUM(keep=TIESUM) sum=TIESUM;
 run;

  proc transpose data=N&U&V out=N1N2(keep=N1 N2) prefix=N;
 var N;
 run;

 data N1N2;
  set N1N2;
  mergeid+1;
  run;

 data W&U&V;
  set W&U&V;
  mergeid+1;
  run;
 
 data TSUM;
  set TSUM;
  mergeid+1;
  run;

 proc sort data=N1N2;
  by MERGEID;
 run;

 proc sort data=W&U&V;
  by MERGEID;
 run;

 proc sort data=TSUM;
  by MERGEID;
 run;

%*Put together Wilcoxon statistics sample sizes and tie information together
  for test statistic calculation. Calculate test statistic;

 data S&U&V;
  merge W&U&V N1N2 TSUM;
  by MERGEID;
  drop MERGEID;

  length DIFFLBL $50;

  NSUM= N1 + N2;
  MIN_N=min(N1,N2);
  NUMER1= _WIL_;
  NUMER2= (MIN_N*(NSUM+1))/2;
  NUMER= NUMER2-NUMER1;
  VAR= ((N1*N2)/24)*(NSUM+1-(TIESUM/(NSUM*(NSUM-1))));
  DENOM=SQRT(VAR);
  W=(NUMER/DENOM); %*Test statistic for Dwass Steele Critchlow-Fligner statistics;
  ABSW=abs(W); %*Absolute value used for two-sided testing;
  QALPHA = probmc('RANGE',.,(1-&ALPHA),.,&NGRPS); %*Studentized-range statistics cutoff;
  if ABSW ge QALPHA then do;
   SYMBOL="**"; %*statistical significance indicator;
  end;

  DIFFLBL="%scan(&GRPVEC,&U," ") - %scan(&GRPVEC,&V," ")";

 run;

 data STAT;
  set STAT S&U&V;
 run; 
 %end;

%end;

data STAT;
 set STAT;
 where N1 ne . and N2 ne .; %*Remove original record from the empty data STAT;
 c+1;

label DIFFLBL="Group@Comparisons"
      W="Test@Statistic"
	  ABSW="Test Statistic@Absolute@Value"
      QALPHA="Cutoff@at@Alpha=&ALPHA" 
	  SYMBOL="Significant@Difference = **"
      c="Comparison@Number"
;

run;

title1 "Large Sample Approximation Multiple Comparison Procedure";
title2 "For All Treatment Pairs";
title3 "Based Upon Pairwise Rankings";
title4 "%trim(%left(&NGRPS)) Groups: %trim(%left(&GRPVEC)) (Respective Sample Sizes: %trim(%left(&NVEC)))";
title5 "Alpha = &ALPHA";
title6 ;
title7 "Method Suggested by Dwass, Steel, Critchlow & Fligner";

%*Print out test statistic results from all pair-wise comparisons;
proc print data=STAT noobs split="@";
var DIFFLBL C W ABSW QALPHA SYMBOL;
run;

%mend DSCF;

%DSCF(TRI,GROUP,TRI,0.05);
