options nodate nonumber;
data PC12;
input GROUP $1-9 DOPA;
cards;
Control    80.32
Control   103.39
Control    96.72
Control    85.05
Control   134.52
Control   105.99
Control   104.19
Control    89.82
Control   128.57
Control    90.48
Control    95.24
Control    95.24
Control    95.24
Control    95.24
StrainI    47.55
StrainI    33.56
StrainI    37.76
StrainI   114
StrainI    70.57
StrainI    54.29
StrainI    84.14
StrainII   51.05
StrainII   60.39
StrainII   68.82
StrainII   77.13
StrainII   53.25
StrainII   41.72
StrainII  119.3
StrainII   79.24
StrainII  131.16
StrainII   98.37
;

options nomacrogen nosymbolgen;

%macro DUNN(DSN,GROUP,VAR,ALPHA);
%***************************************************************;
%* The Dunn Macro                                              *;
%* ------------------------------------------------------------*;
%*                                                             *;
%* This macro is designed to perform all two-sided pair-wise   *;
%* comparisons of the location parameters for n groups (n gt 2)*;
%* in a one-way layout. The method for the procedure involves a*;
%* large sample approximation. The method of comparison employs*;
%* ranking all non-missing observations and averaging the ranks*;
%* for each level of the class variable (&GROUP).              *;
%*                                                             *;
%* The required inputs for the macro are a SAS data set (&DSN) *;
%* containing a character class variable (&GROUP), a response  *;
%* variable (&VAR), and a family-wise error rate (&ALPHA).     *;
%*                                                             *;
%* For more details on this nonparametric multiple comparison  *;
%* procedure, see Hollander and Wolfe s Nonparametric          *;
%* Statistical Methods, 1/e p. 125.                            *;
%*                                                             *;
%* Overall Macro Scheme:                                       *;
%* --------------------                                        *;
%* The macro consists of a body of code containing one embedded*;
%* macro (%GROUPS). The embedded macro determines the number of*;
%* groups present (&NGRPS). If a group in the SAS data set does*;
%* not contain at least one response value, it will not be     *;
%* included in the analysis. The embedded macro also creates   *;
%* one global macro variable that contains the group labels    *;
%* (&GRPVEC) for the levels of the class variable.             *;
%*                                                             *;
%* The main body of the SAS macro code determines summary      *;
%* statistics (e.g., average ranks, sample sizes, etc.) This   *;
%* information is then employed to calculate the pair-wise     *;
%* test statistics. The cutoff for the test statistic is       *;
%* calculated with PROBIT function. The results are then       *;
%* printed out with a PROC PRINT procedure.                    *;
%*                                                             *;
%*If you have questions about this code or its application,    *;
%*please do not hesitate to contact me.                        *;
%*                                                             *;
%*Paul Juneau                                                  *;
%*Associate Director                                           *;
%*Midwest Nonclinical Statistics - Michigan Laboratories       *;
%*Pfizer Global Research & Development                         *;
%*734-622-1791                                                 *;
%*paul.juneau@pfizer.com                                       *;
%*                                                             *;
%*2004                                                         *;
%***************************************************************;

%*First, take the input data set &DSN and eliminate missing values;
data &DSN.2;
 set &DSN;
 if &VAR ne .;
run;

%macro GROUPS;
%*This macro creates two outputs:                         *;
%*(1) The total number of groups present (&NGRPS)         *; 
%*(2) A macro variable that contains all of the individual*;
%*    class variable labels (&GRPVEC)                     *;


%global GRPVEC;

proc sort data=&DSN.2; by &GROUP;
run;

%*Create a data set that contains just the levels of the class variable (&GROUP);
data &GROUP; set &DSN.2; by &GROUP;
if first.&GROUP;
run;

proc transpose data=&GROUP out=GRPVEC; var &GROUP;
run;

%*Determine the number of levels of the class variable;
data _null_;
call symput('NGRPS',left(put(count,8.)));
stop;
set &GROUP nobs=COUNT;
run;

%*Create a global macro variable containing the labels for each level of the class variable;
data null; set GRPVEC;

GRPVEC=
%do G=1 %to %eval(&NGRPS-1);
 trim(left(COL&G))||" "||
%end;
 trim(left(COL&NGRPS));
call SYMPUT('GRPVEC',GRPVEC);
run;

%mend GROUPS;

%GROUPS

proc sort data=&DSN.2;
by &GROUP;
run;

%*Rank all non-missing responses;
proc rank data=&DSN.2 out=R&VAR;
ranks R&VAR;
var &VAR;
run;

proc sort data=R&VAR;
by &GROUP;
run;

%*Calculate the average of the ranks for each group;
proc univariate noprint data=R&VAR;
var R&VAR;
by &GROUP;
output out=S&VAR sum=S&VAR n=N&VAR;
run;

%*Determine the total sample size for the entire experiment;
proc univariate noprint data=S&VAR;
var N&VAR;
output out=SN&VAR sum=SN&VAR;
run;

%*Create a macro variable with value equal to the total sample size for
  the entire experiment;

data _null_;
 set SN&VAR;

call symput("N",left(put(SN&VAR,8.)));
run;

data SR&VAR; set S&VAR;
keep S&VAR;
run;

data N&VAR; set S&VAR;
keep N&VAR;
run;

proc transpose data=SR&VAR out=SRVEC; var S&VAR;
run;

%*Create a macro variable containing the summary statistics (average
  ranks) for each group;

data _null_;
 set SRVEC;
SRVEC=
%do S=1 %to %eval(&NGRPS-1);
 trim(left(put(COL&S,11.3)))||" "||
%end;
 trim(left(put(COL&NGRPS,11.3)));
call SYMPUT('SRVEC',SRVEC);
run;

proc transpose data=N&VAR out=nVEC; var n&VAR;
run;

%*Create a macro variable containing the sample sizes for each level
  of the class variable;

data _null_;
 set NVEC;
nVEC=
%do S2=1 %to %eval(&NGRPS-1);
 trim(left(put(COL&S2,8.)))||" "||
%end;
 trim(left(put(COL&NGRPS,8.)));
call SYMPUT('NVEC',NVEC);
run;

%*Perform the pair-wise large sample approximation MCP;
data NPARMCP;

%do U=1 %to &NGRPS;

 %do V=%eval(&U+1) %to &NGRPS;

  %let RSUM&U=%scan(&SRVEC,&U," ");
  %let N&U=%scan(&NVEC,&U," ");
  %let RSUM&V=%scan(&SRVEC,&V," ");
  %let N&V=%scan(&NVEC,&V," ");

  c+1;
  DIFFLBL="%scan(&GRPVEC,&U," ") - %scan(&GRPVEC,&V," ")";
  AVDIFF=abs((&&RSUM&U/&&N&U)-(&&RSUM&V/&&N&V));
  CUTOFF=probit(1-(&ALPHA/((&NGRPS*(&NGRPS-1)))))
         *(((&N*(&N+1))/12)**0.5)
         *((1/&&N&U + 1/&&N&V)**0.5);
  SYMBOL="  ";
  if AVDIFF ge CUTOFF then do;
   SYMBOL="**";
  end;
  output;
 %end;

%end;

label DIFFLBL="Group@Comparisons"
      AVDIFF="Difference@in@Average@Ranks"
      CUTOFF="Cutoff@at@Alpha=&ALPHA"
      SYMBOL="Significance@Difference = **"
      c="Comparison@Number"
;

title1 "Large Sample Approximation Multiple Comparison Procedure";
title2 "Designed for Unbalanced Data";
title3 "%trim(%left(&NGRPS)) Groups: %trim(%left(&GRPVEC)) (Respective Sample Sizes: %trim(%left(&NVEC)))";
title4 "Alpha = &ALPHA";
title5 ;
title6 "Method Suggested by Dunn (1964)";

%*Print out the results of the test;
proc print label split='@' noobs;
run;

%mend DUNN;

%DUNN(PC12,GROUP,DOPA,0.05);
